package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;

public class Ball extends Entity implements PhysicsBody, CollisionListener {

	public static final float RADIUS = 1.2f;
	public static final Color BALL_COLOR = Color.CHARTREUSE;
	public static final float DEFAULT_SPEED = 45;
	public static final float BOOST_SPEED = 55;
	public static final int BLOCKS_FOR_BOOST = 39;

	private float speed;
	private boolean isStuck = true;
	private boolean touchedPaddle = false;

	private Body body;

	public Ball(PlayState state) {
		super(state, state.paddle.getX(), state.paddle.getY() + Paddle.PADDLE_HEIGHT + RADIUS);
		this.speed = state.bricks.size > BLOCKS_FOR_BOOST? DEFAULT_SPEED : BOOST_SPEED;
		createBody();
	}

	@Override
	public void render(ShapeRenderer sr) {
		sr.begin(ShapeType.Filled);
		sr.setColor(BALL_COLOR);
		sr.circle(pos.x * PlayState.PIXEL_PER_METER,
				pos.y * PlayState.PIXEL_PER_METER,
				RADIUS * PlayState.PIXEL_PER_METER);
		sr.end();
	}

	@Override
	public void update(float dt) {
		if (isStuck || deleted) {
			if (!deleted && Gdx.input.justTouched()) {
				launch();
			} else {
				return;
			}
		}
		if (touchedPaddle) {
			paddleCollision();
			touchedPaddle = false;
		}

		if (getY() + RADIUS < 0) {
			deleted = true;
		}

		body.setLinearVelocity(body.getLinearVelocity().nor().scl(speed));
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef ballBody = new BodyDef();
		ballBody.type = BodyDef.BodyType.DynamicBody;
		ballBody.position.set(pos);

		CircleShape ballShape = new CircleShape();
		ballShape.setRadius(RADIUS);

		FixtureDef ballFixture = new FixtureDef();
		ballFixture.shape = ballShape;
		ballFixture.restitution = 1f;
		ballFixture.friction = 0f;
		ballFixture.density = 1f;

		body = state.world.createBody(ballBody);
		body.createFixture(ballFixture);
		body.setUserData(this);

		ballShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Shield) {
			contacted.delete();
		}

		if (contacted instanceof Paddle && !isStuck) {
			touchedPaddle = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {
	}

	public Vector2 paddleReflectAngle() {
		float trim = state.paddle.getWidth() * 0.10f;
		float rel = MathUtils.clamp((pos.x - state.paddle.getX()) + (state.paddle.getWidth()/2),
				trim, state.paddle.getWidth()-trim);
		float newAngle = MathUtils.PI - (MathUtils.PI * (rel / state.paddle.getWidth()));
		return new Vector2(MathUtils.cos(newAngle), MathUtils.sin(newAngle));
	}

	public void launch() {
		Vector2 direction;
		if (state.paddle.isSticky()) {
			direction = paddleReflectAngle();
		} else {
			// launch at random angle between 135 and 45 degrees
			float angle = MathUtils.random(MathUtils.PI/2) + MathUtils.PI/4;
			direction = new Vector2(MathUtils.cos(angle), MathUtils.sin(angle));

		}
		body.setLinearVelocity(direction.scl(speed));
		isStuck = false;
	}

	public void paddleCollision() {
		if (state.paddle.isSticky()) {
			isStuck = true;
			body.setLinearVelocity(new Vector2());
			setY(state.paddle.getY() + Paddle.PADDLE_HEIGHT + RADIUS);
			return;
		}
		body.setLinearVelocity(paddleReflectAngle().scl(speed));
	}

	@Override
	public void setX(float x) {
		super.setX(x);
		Vector2 bodyPos = body.getPosition();
		bodyPos.x = x;
		body.setTransform(bodyPos, 0);
	}

	@Override
	public void setY(float y) {
		super.setY(y);
		Vector2 bodyPos = body.getPosition();
		bodyPos.y = y;
		body.setTransform(bodyPos, 0);
	}

	public void setSpeed(float speed) {
		this.speed = speed;
	}

	public boolean isStuck() {
		return isStuck;
	}

}
